@@ -24,11 +24,11 @@ if '__main__' == __name__:  # pragma: no cover
 
 try:
     from pageit import tools
-    from pageit.namespace import Namespace
+    from pageit.namespace import Namespace, DeepNamespace
     import pageit
 except ImportError:  # pragma: no cover
     from . import tools
-    from .namespace import Namespace
+    from .namespace import Namespace, DeepNamespace
     import __init__ as pageit  # pylint: disable=W0403
 
 logging.basicConfig(format='%(levelname)-8s %(message)s')
@@ -106,13 +106,14 @@ class Pageit(object):
         tmpl (mako.lookup.TemplateLookup, optioanl): mako template lookup
             object
 
-        site (pageit.namespace.Namespace, optional): namespace passed to
-            mako templates during rendering
+        site (pageit.namespace.DeepNamespace, optional):
+            :py:class:`~pageit.namespace.DeepNamespace` passed to mako
+            templates during rendering
 
         log (logging.Logger, optional): system logger
 
     .. versionchanged:: 0.2.1
-       Added the *site* parameter.
+       Added the ``site`` parameter.
     '''
 
     _dry = ''
@@ -155,10 +156,11 @@ class Pageit(object):
             str: next file to process
         '''
         pattern = '*' + self.args.ext
-        for relpath, _, files in os.walk(self.path):
+        for relpath, dirs, files in os.walk(self.path):
             src = osp.join(self.path, relpath)
-            if fnmatch(src, pattern):  # don't go into this directory
-                continue
+            for i, name in enumerate(dirs):
+                if fnmatch(name, pattern):
+                    del dirs[i]  # don't visit that directory
 
             for name in files:
                 if fnmatch(name, pattern):  # do list this file
@@ -201,6 +203,16 @@ class Pageit(object):
 
         Returns:
             Pageit: for method chaining
+
+        Examples:
+            >>> import os.path as osp
+            >>> runner = Pageit('test/example1')
+            >>> _ = runner.run()
+            >>> _ = runner.on_change('test/example1/index.html.mako')
+            >>> _ = runner.on_change(osp.abspath('test/example1/index.html'))
+            >>> _ = runner.clean()
+            >>> _ is runner
+            True
         '''
         if path in self._outputs:
             return self
@@ -212,6 +224,19 @@ class Pageit(object):
 
         Returns:
             Pageit: for method chaining
+
+        Examples:
+            >>> runner = Pageit('test/example1', dry_run=True)
+            >>> runner.run().clean() is runner
+            True
+
+            >>> runner = Pageit('test/example1', ignore_mtime=True)
+            >>> runner.run().clean() is runner
+            True
+
+            >>> runner = Pageit('test/example1')
+            >>> runner.run().clean() is runner
+            True
         '''
         _context = '[RENDER]'
         self.log.debug(MSG.START, _context)
@@ -248,12 +273,28 @@ class Pageit(object):
     def mako(self, path, dest=None):
         '''Render a mako template.
 
+        .. _special-mako-vars:
+
+        This function injects two :py:class:`~pageit.namespace.DeepNamespace`
+        variables into the ``mako`` template:
+
+        - ``site``: environment information passed into the constructor
+        - ``page``: information about the current template
+
+          - ``path``: relative path of this template
+          - ``output``: relative path to the output of the template
+          - ``dirname``: name of the directory containing the template
+          - ``basedir``: relative path back to the root directory
+
         Args:
             path (str): template path
             dest (str, optional): output path; if not provided will be computed
 
         Returns:
             Pageit: for method chaining
+
+        .. versionchanged:: 0.2.2
+           Added more template information (output, dirname, basedir).
         '''
         _context = '[MAKO]'
         name = osp.relpath(path, self.path)
@@ -262,7 +303,12 @@ class Pageit(object):
         dest = dest or strip_ext(path, self.args.ext)
         tmpl = self.tmpl.get_template(name)
         content, has_errors = '', False
-        page = Namespace(path=name)
+        page = DeepNamespace(
+            path=name,
+            output=osp.relpath(dest, self.path),
+            dirname=osp.dirname(name),
+            basedir=osp.relpath(self.path, osp.dirname(path))
+        )
         try:
             if not self.args.dry_run:
                 content = tmpl.render_unicode(site=self.site, page=page)
@@ -271,7 +317,7 @@ class Pageit(object):
             has_errors = True
             self.log.error(MSG.RENDER_ERR, _context, path)
             self.log.error(ex)
-            if not self.args.noerr:
+            if not self.args.dry_run and not self.args.noerr:
                 content = mako.exceptions.html_error_template().render()
 
         if not (self.args.noerr and has_errors):
@@ -283,7 +329,7 @@ class Pageit(object):
                     self._outputs.append(dest)
                 self.log.debug(MSG.WRITE + self._dry, _context,
                                osp.relpath(dest, self.path))
-            except OSError as ex:
+            except OSError as ex:  # pragma: no cover
                 self.log.error(MSG.WRITE_ERR, _context, dest, ex)
 
         self.log.debug(MSG.DONE, _context)
@@ -300,6 +346,10 @@ class Pageit(object):
 
         Returns:
             set: paths of dependencies
+
+        Examples:
+            >>> Pageit().mako_deps('fake.mako')
+            set([])
         '''
         paths = set([])
         if not osp.isfile(path):
@@ -335,6 +385,12 @@ class Pageit(object):
         Examples:
             >>> Pageit().mako_mtime('fake.mako')
             0
+
+            >>> import os.path as osp
+            >>> path1 = osp.abspath('test/example1')
+            >>> path2 = osp.join(path1, 'subdir/test-page.html.mako')
+            >>> Pageit(path1).mako_mtime(path2) > 0
+            True
         '''
         _context = '[MTIME]'
         name = osp.relpath(path, self.path)
@@ -346,7 +402,7 @@ class Pageit(object):
         deps, next_deps, done, mtimes = set([path]), set([]), [], []
         for _ in range(levels + 1):
             for dep in deps:
-                if dep in done or dep in next_deps or not osp.isfile(dep):
+                if not osp.isfile(dep):
                     continue
 
                 next_deps = next_deps.union(self.mako_deps(dep))
@@ -375,7 +431,16 @@ def create_logger(verbosity=DEFAULT.verbosity, log=None):
         logging.Logger: logger configured based on verbosity.
 
     Example:
-        >>> create_logger() is not None
+        >>> log = create_logger()
+        >>> log.getEffectiveLevel() == logging.INFO
+        True
+
+        >>> log = create_logger(0, log)
+        >>> log.getEffectiveLevel() == logging.NOTSET
+        True
+
+        >>> log = create_logger(2, log)
+        >>> log.getEffectiveLevel() == logging.DEBUG
         True
     '''
     log = log or DEFAULT.log
@@ -411,36 +476,46 @@ def create_lookup(path=DEFAULT.path, tmp=None):
 
 
 def create_config(path=DEFAULT.config, env=DEFAULT.env, log=None):
-    '''Constructs a Namespace for attributes to pass to mako templates.
+    '''Constructs a :py:class:`~pageit.namespace.DeepNamespace` for attributes
+    to pass to mako templates.
 
     The configuration file should be a list of keys mapped to key/value pairs.
     The load process first loads an environment called "default" and then
     extends those keys to include those in the given environment.
 
-    .. versionadded:: 0.2.1
-
     Args:
         path (str): YAML configuration file
         env (str, optional): section to load
         log (logging.Logger, optional): system logger
 
     Returns:
-        pageit.namespace.Namespace: namespace of environment attributes
+        pageit.namespace.DeepNamespace:
+        :py:class:`~pageit.namespace.DeepNamespace` of environment attributes
 
     Examples:
-        >>> create_config('file.yml', 'local') == Namespace()
+        >>> create_config('file.yml', 'local') == DeepNamespace()
         True
+
+        >>> conf = create_config('test/example1/pageit.yml', 'test')
+        >>> conf.debug == True
+        True
+
+        >>> conf = create_config('test/example1/pageit.yml', 'fakeenv')
+        >>> 'debug' not in conf
+        True
+
+    .. versionadded:: 0.2.1
     '''
     _context = '[CONFIG]'
     log = log or create_logger()
-    result = Namespace()
+    result = DeepNamespace()
     if not osp.isfile(path):
         log.warning(MSG.PATH_ERR, _context, path)
         return result
 
-    all_env = Namespace(yaml.load(open(path)))
+    all_env = DeepNamespace(yaml.load(open(path)))
     if DEFAULT.env in all_env:
-        log.debug(MSG.LOAD_ENV, _context, 'default', path)
+        log.debug(MSG.LOAD_ENV, _context, DEFAULT.env, path)
         result += all_env[DEFAULT.env]
 
     if DEFAULT.env != env:
@@ -500,7 +575,7 @@ def render(args):  # pragma: no cover
     '''
     args.path = osp.abspath(args.path)
     log = create_logger(args.verbosity)
-    site = Namespace(_pageit=Namespace(version=pageit.__version__))
+    site = DeepNamespace(_pageit=DeepNamespace(version=pageit.__version__))
     tmpl = create_lookup(args.path, args.tmp)
 
     if not osp.isfile(args.config):  # adjust relative to path
